//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "Group.hpp"
//------------------------------------------------------------------------------
#include <iostream>
//------------------------------------------------------------------------------
#include "MyException.hpp"
#include "Screen.hpp"
#include "AllObj.hpp"

#include "glUtils.hpp"

#include <lua.hpp>
#include <luabind/luabind.hpp>
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
using namespace std;
//------------------------------------------------------------------------------
bool compareZpos(const Drawable *d1, const Drawable *d2)
{
    return (d1->GetZpos() < d2->GetZpos());
}
//------------------------------------------------------------------------------
Group::Group(Group *parent, const std::string &name) : Drawable(parent, name), m_scale(Vector2D(1.f, 1.f)), m_cutRect(sf::FloatRect(0, 0, 0, 0)), m_isCut(false)
{
    //ctor
}
//------------------------------------------------------------------------------
Group::~Group()
{
    Clear();
}
//------------------------------------------------------------------------------
void Group::DestroyChildren()
{
    Screen *parentScreen = GetParentScreenRaw();

    // We need to notice the children
    for(DrawZPosMap::iterator it = m_drawTbl.begin(); it != m_drawTbl.end(); it++ )
    {
        for(DrawLowZPosMap::iterator it2 = it->second.begin(); it2 != it->second.end(); it2++ )
        {
            //it2->second->Destroy();

            it2->second->OnChangeScreen(parentScreen, NULL);
            delete it2->second;
        }
    }
    m_drawTbl.clear();

    for(std::vector<PhysicObject*>::iterator it = m_phTbl.begin(); it != m_phTbl.end(); it++ )
    {
        (*it)->Destroy();
        (*it)->OnChangeScreen(parentScreen, NULL);
        delete *it;
    }
    m_phTbl.clear();
}
//------------------------------------------------------------------------------
int Group::GetChildCount() const
{
    return GetPhysicChildCount() + GetDrawableChildCount();
}
//------------------------------------------------------------------------------
/** Return the number of physic children that this group contains */
int Group::GetPhysicChildCount() const
{
    return m_phTbl.size();
}
//------------------------------------------------------------------------------
/** Return the number of drawable children that this group contains */
int Group::GetDrawableChildCount() const
{
    int count = 0;

    for(DrawZPosMap::const_iterator it = m_drawTbl.begin(); it != m_drawTbl.end(); it++ )
    {
        count += it->second.size();
    }

    return count;
}
//------------------------------------------------------------------------------
/** Return the physic child with the given index */
PhysicObject *Group::GetPhysicChildUncast(int index)
{
    if(index < 0 || index >= GetPhysicChildCount())
    {
        MY_THROW("Group::GetPhysicChild() : out of range !")
    }

    return m_phTbl[index];
}
//------------------------------------------------------------------------------
/** Return the physic child with the given index but do not cast it into luabind::object */
luabind::object Group::GetPhysicChild(int index)
{
    return CastObject(GetPhysicChildUncast(index));
}
//------------------------------------------------------------------------------
/** Return the drawable with the given index but do not cast it into luabind::object */
luabind::object Group::GetDrawableChild(int index)
{
    return CastObject(GetDrawableChildUncast(index));
}
//------------------------------------------------------------------------------
/** Return the drawable with the given index */
Drawable *Group::GetDrawableChildUncast(int index)
{
    if(index < 0)
    {
        MY_THROW("Group::GetDrawableChild() : out of range !")
    }

    size_t pos = 0;
    for(DrawZPosMap::const_iterator it = m_drawTbl.begin(); it != m_drawTbl.end(); it++ )
    {
        const size_t size = it->second.size();

        if(size_t(index) < pos + size)
        {
            int currentPos = pos;

            for(DrawLowZPosMap::const_iterator it2 = it->second.begin(); it2 != it->second.end(); it2++ )
            {
                if(currentPos == index)
                {
                    return (it2->second);
                }
                currentPos++;
            }
        }

        pos += size;
    }

    MY_THROW("Group::GetDrawableChild() : out of range !")
}
//------------------------------------------------------------------------------
void Group::RegisterClone(Group *newParent)
{
    // We change the parent if it's needed
    if(newParent != NULL)
    {
        m_parent = newParent;
    }

    // /!\ Specific to Drawable object : we need to refresh the lowPos
    m_lowZpos = GetParentRaw()->Add(this);

    // We notice the object that the parent has changed => Will attach the objet to the pysic world
    // !! Already done in constructor !
    //OnChangeScreen(NULL, GetParentScreenRaw());
}
//------------------------------------------------------------------------------
Group::Group(const Group &grp, Group *newParent) : Drawable(grp)
{
    if(newParent != NULL)
    {
        m_parent = newParent; // We need to set the new parent at first in the group so that physic objects won't register in any other group.
    }

    // Do not copy m_luaPreDestroyFunction

    m_scale = grp.m_scale;
    m_isCut = grp.m_isCut;
    m_cutRect = grp.m_cutRect;

    m_drawTbl.clear();

    for(DrawZPosMap::const_iterator it = grp.m_drawTbl.begin(); it != grp.m_drawTbl.end(); it++ )
    {
        for(DrawLowZPosMap::const_iterator it2 = it->second.begin(); it2 != it->second.end(); it2++ )
        {
            it2->second->GetRawCloneToGroup(this);
        }
    }

    m_phTbl.clear();
    // iterators are invalidated for vectors
    for(std::vector<PhysicObject*>::const_iterator it = grp.m_phTbl.begin(); it != grp.m_phTbl.end(); it++)
    {
        (*it)->GetRawCloneToGroup(this);
    }
}
//------------------------------------------------------------------------------
/** Create a copy of a group but do not copy its elements */
Group::Group(const Group &grp) : Drawable(grp)
{
    // Do not copy m_luaPreDestroyFunction

    m_scale = grp.m_scale;
    m_isCut = grp.m_isCut;
    m_cutRect = grp.m_cutRect;
}
//------------------------------------------------------------------------------
void Group::OnChangeScreen(Screen *backScreen, Screen *newScreen)
{
    if(backScreen != newScreen/* && backScreen != NULL && newScreen != NULL*/) // If we don't change screen, this is unusefull
    {
        // We need to notice the children

        for(DrawZPosMap::const_iterator it = m_drawTbl.begin(); it != m_drawTbl.end(); it++ )
        {
            for(DrawLowZPosMap::const_iterator it2 = it->second.begin(); it2 != it->second.end(); it2++ )
            {
                it2->second->OnChangeScreen(backScreen, newScreen);
            }
        }

        for(std::vector<PhysicObject*>::const_iterator it = m_phTbl.begin(); it != m_phTbl.end(); it++ )
        {
            (*it)->OnChangeScreen(backScreen, newScreen);
        }
    }
}
//------------------------------------------------------------------------------
// FindByName
#define X(typeName, typeId) \
        ProtectedPtr<typeName> Group::FindByName ## typeName(const std::string &name) \
        { \
            Object *obj = FindByNameUncast(name); \
            if(obj->GetType() == typeId) \
            { \
                return ProtectedPtr<typeName>(static_cast<typeName*>(obj)); \
            } \
            else \
            { \
                MY_THROW("Group::FindByName"# typeName "() : can't cast '" << name << "', bad type !"); \
            } \
        }
    X_OBJ_LIST
#undef X
//------------------------------------------------------------------------------
ProtectedPtr<Group> Group::CreateGroup(const std::string &name)
{
    return ProtectedPtr<Group>(new Group(this, name));
}
//------------------------------------------------------------------------------
ProtectedPtr<Group> Group::CreateGroup(const std::string &name, const Vector2D &position)
{
    ProtectedPtr<Group> obj(new Group(this, name));
    obj->SetPosition(position);
    return obj;
}
//------------------------------------------------------------------------------
ProtectedPtr<ParticleSource> Group::CreateParticleSource(const std::string &name, const ParticleModel &model, int maxParticleNumber)
{
    return new ParticleSource(this, name, model, maxParticleNumber);
}
//------------------------------------------------------------------------------
ProtectedPtr<Shape> Group::CreateShape(const std::string &name)
{
    return new Shape(this, name);
}
//------------------------------------------------------------------------------
ProtectedPtr<Sprite> Group::CreateSprite(const std::string &name)
{
    return new Sprite(this, name);
}
//------------------------------------------------------------------------------
ProtectedPtr<Sprite> Group::CreateSprite(const std::string &name, Texture &image, const Vector2D &position)
{
    ProtectedPtr<Sprite> obj(new Sprite(this, name));
    obj->SetPosition(position);
    obj->SetImage(image);
    return obj;
}
//------------------------------------------------------------------------------
ProtectedPtr<Text> Group::CreateText(const std::string &name, const std::string &text)
{
    return new Text(this, name, text);
}
//------------------------------------------------------------------------------
ProtectedPtr<Text> Group::CreateText(const std::string &name, const std::string &text, int fontSize, const Color &fontColor, const Vector2D &position)
{
    ProtectedPtr<Text> obj(new Text(this, name, text));
    obj->SetPosition(position);
    obj->SetColor(fontColor);
    obj->SetSize(fontSize);
    return obj;
}
//------------------------------------------------------------------------------
ProtectedPtr<TexturedArea> Group::CreateTexturedArea(const std::string &name, Texture &image, const Vector2D &position, const Vector2D &areaSize)
{
    ProtectedPtr<TexturedArea> obj(new TexturedArea(this, name));
    obj->SetPosition(position);
    obj->SetImage(image);
    obj->SetAreaSize(areaSize);
    return obj;
}
//------------------------------------------------------------------------------
ProtectedPtr<PostFX> Group::CreatePostFX(const std::string &name)
{
    return new PostFX(this, name);
}
//------------------------------------------------------------------------------
ProtectedPtr<ScreenRenderer> Group::CreateScreenRenderer(const std::string &name)
{
    return new ScreenRenderer(this, name);
}
//------------------------------------------------------------------------------
ProtectedPtr<PhLine> Group::CreatePhLine(int phGroupId, const std::string &name, const Vector2D &pt1, const Vector2D &pt2)
{
    return new PhLine(this, phGroupId, name, pt1, pt2);
}
//------------------------------------------------------------------------------
ProtectedPtr<PhRect> Group::CreatePhRect(int phGroupId, const std::string &name, const Vector2D &size)
{
    return new PhRect(this, phGroupId, name, size);
}
//------------------------------------------------------------------------------
ProtectedPtr<PhRect> Group::CreatePhRect(int phGroupId, const std::string &name, const Vector2D &position, const Vector2D &size)
{
    ProtectedPtr<PhRect> obj(new PhRect(this, phGroupId, name, size));
    obj->SetPosition(position);
    return obj;
}
//------------------------------------------------------------------------------
ProtectedPtr<PhTriangle> Group::CreatePhTriangle(int phGroupId, const std::string &name, const Vector2D &pt1, const Vector2D &pt2, const Vector2D &pt3)
{
    return new PhTriangle(this, phGroupId, name, pt1, pt2, pt3);
}
//------------------------------------------------------------------------------
ProtectedPtr<PhShape> Group::CreatePhShape(int phGroupId, const std::string &name)
{
    return new PhShape(this, phGroupId, name);
}
//------------------------------------------------------------------------------
ProtectedPtr<PhCircle> Group::CreatePhCircle(int phGroupId, const std::string &name, const Vector2D &center, float radius)
{
    return new PhCircle(this, phGroupId, name, center, radius);
}
//------------------------------------------------------------------------------
void Group::SetScale(const Vector2D &scale)
{
    m_scale = scale;
}
//------------------------------------------------------------------------------
Vector2D Group::GetScale()
{
    return m_scale;
}
//------------------------------------------------------------------------------
// Call pre destroy lua function
void Group::CallPreDestroyLuaFunction()
{
    // Event pre destroy
    const luabind::object &luaFunction = m_luaPreDestroyFunction;

    if(luaFunction.is_valid())
    {
        try
        {
            luabind::call_function<void>(luaFunction, ProtectedPtr<Group>(this));
        }
        catch(luabind::error &e)
        {
            cerr << "Group::LuaPreDestroyFunction(): Failed to run lua function !" << endl;
            cerr << "LUA error (error) :\n" << lua_tostring(Global::GetLuaState(), -1) << endl;
            cerr << "What: " << e.what() << endl;
            cerr << "File : '" << __FILE__ << "' at line " << __LINE__ << endl;
            cerr << "Group name : '" << GetName() << "'." << endl;
        }
        catch(luabind::cast_failed &e)
        {
            cerr << "Group::LuaPreDestroyFunction(): Failed to run lua function !" << endl;
            cerr << "LUA error (cast_failed) :\n" << lua_tostring(Global::GetLuaState(), -1) << endl;
            cerr << "What: " << e.what() << endl;
            cerr << "File : '" << __FILE__ << "' at line " << __LINE__ << endl;
            cerr << "Group name : '" << GetName() << "'." << endl;
        }
        catch(std::exception &e)
        {
            cerr << "Group::LuaPreDestroyFunction(): Failed to run lua function (std::exception) !" << endl;
            cerr << "What: " << e.what() << endl;
            cerr << "File : '" << __FILE__ << "' at line " << __LINE__ << endl;
            cerr << "Group name : '" << GetName() << "'." << endl;
        }
    }

    m_luaPreDestroyFunction = luabind::object(); // nil
}
//------------------------------------------------------------------------------
void Group::Clear()
{
    // Call pre-destroy fn
    CallPreDestroyLuaFunction();

    // Drawable
    for(DrawZPosMap::iterator it = m_drawTbl.begin(); it != m_drawTbl.end(); it++ )
    {
        for(DrawLowZPosMap::iterator it2 = it->second.begin(); it2 != it->second.end(); it2++ )
        {
            delete it2->second;
        }
        it->second.clear();
    }
    m_drawTbl.clear();

    // Physic objects
    for(std::vector<PhysicObject*>::iterator it = m_phTbl.begin(); it != m_phTbl.end(); it++ )
    {
        delete *it;
    }
    m_phTbl.clear();
}
//------------------------------------------------------------------------------
void Group::Render(sf::RenderTarget &App)
{
    const sf::View &backView = App.GetView();
    sf::View tmpView;
    GlCut *glCut = NULL;

    // Scale
    if(m_scale != Vector2D(1.f, 1.f))
    {
        sf::FloatRect backRect = backView.GetRect();
        Vector2D globalPos = GetGlobalPosition();

        sf::FloatRect viewRect;
        viewRect.Left = (globalPos.x - backRect.Left) * ((m_scale.x - 1.f) / m_scale.x) + backRect.Left;
        viewRect.Top = (globalPos.y - backRect.Top) * ((m_scale.y - 1.f) / m_scale.y) + backRect.Top;
        viewRect.Bottom = backRect.GetHeight() / m_scale.x + viewRect.Top;
        viewRect.Right = backRect.GetWidth() / m_scale.y + viewRect.Left;
        tmpView.SetFromRect(viewRect);
        App.SetView(tmpView);
    }

    // Draw
    if(IsVisible())
    {
        // Cut
        if(m_isCut)
        {
            Vector2D globalPos = GetGlobalPosition();
            glCut = new GlCut(sf::FloatRect(m_cutRect.Left + globalPos.x, m_cutRect.Top + globalPos.y, m_cutRect.Right + globalPos.x, m_cutRect.Bottom + globalPos.y), App);
        }

        // Render all drawables
        for(DrawZPosMap::iterator it = m_drawTbl.begin(); it != m_drawTbl.end(); it++ )
        {
            for(DrawLowZPosMap::iterator it2 = it->second.begin(); it2 != it->second.end(); it2++ )
            {
                it2->second->Render(App);
            }
        }

        // Cut
        if(glCut)
        {
            delete glCut;
        }
    }

    // Debug
    if(Global::IsShowPhysicObjects())
    {
        // Group center
        Vector2D pos = GetGlobalPosition();
        sf::Shape circle = sf::Shape::Circle(pos.x, pos.y, 4, Color(0, 255, 0));
        App.Draw(circle);

        // Physic objects
        for(std::vector<PhysicObject*>::iterator it = m_phTbl.begin(); it != m_phTbl.end(); it++ )
        {
            (*it)->RenderShape(App);
        }
    }

    App.SetView(backView);
}
//------------------------------------------------------------------------------
int Group::Add(Drawable *obj)
{
    int lowZPos = 0;
    DrawLowZPosMap &map = m_drawTbl[obj->GetZpos()];
    if(!map.empty())
    {
        DrawLowZPosMap::iterator last = map.end();
        last --; // We use the low id of the last element
        lowZPos = last->first + 1; // The new lowZPos is the last lowzpos of the list + 1
    }

    map[lowZPos] = obj;
    return lowZPos;
}
//------------------------------------------------------------------------------
void Group::Add(PhysicObject *obj)
{
    m_phTbl.push_back(obj);
}
//------------------------------------------------------------------------------
void Group::Remove(Drawable *obj)
{
    bool found = false;

    DrawLowZPosMap &map = m_drawTbl[obj->GetZpos()];
    DrawLowZPosMap::iterator it = map.find(obj->GetLowZPos());
    if(it->second == obj)
    {
        map.erase(it);
        found = true;
    }

    if(!found)
    {
        MY_THROW("Group::Remove : Pointer of object '" << obj->GetName() << "' not found !");
    }
}
//------------------------------------------------------------------------------
void Group::Remove(PhysicObject *obj)
{
    std::vector<PhysicObject*>::iterator it = find(m_phTbl.begin(), m_phTbl.end(), obj);
    if(it != m_phTbl.end())
    {
        m_phTbl.erase(it);
    }
    else
    {
        MY_THROW("Group::Remove : Pointer not found !");
    }
}
//------------------------------------------------------------------------------
void Group::ProceedMvt(float time)
{
    // This
    Drawable::ProceedMvt(time);

    // Drawable
    for(DrawZPosMap::iterator it = m_drawTbl.begin(); it != m_drawTbl.end();)
    {
        DrawZPosMap::iterator nextIt = it;
        nextIt++;

        // The entire low zpos map could be destroyed if it becomes empty
        DrawLowZPosMap lowZposMapCopy = it->second;
        for(DrawLowZPosMap::iterator it2 = lowZposMapCopy.begin(); it2 != lowZposMapCopy.end(); it2++)
        {
            it2->second->ProceedMvt(time);
        }

        it = nextIt;
    }

    // Physic
    for(std::vector<PhysicObject*>::iterator it = m_phTbl.begin(); it != m_phTbl.end(); it++ )
    {
        (*it)->ProceedMvt(time);
    }
}

//------------------------------------------------------------------------------
ProtectedPtr<Shape> Group::CreateShapeLine(const std::string &name, const Vector2D& P1, const Vector2D& P2, float Thickness, const Color& Col, float Outline, const Color& OutlineCol)
{
	ProtectedPtr<Shape> shape = CreateShape(name);
	shape->MakeLine(P1, P2, Thickness, Col, Outline, OutlineCol);
	return shape;
}
//------------------------------------------------------------------------------
ProtectedPtr<Shape> Group::CreateShapeRectangle(const std::string &name, const Vector2D& P1, const Vector2D& P2, const Color& Col, float Outline, const Color& OutlineCol)
{
	ProtectedPtr<Shape> shape = CreateShape(name);
	shape->MakeRectangle(P1, P2, Col, Outline, OutlineCol);
	return shape;
}
//------------------------------------------------------------------------------
ProtectedPtr<Shape> Group::CreateShapeRectangleSize(const std::string &name, const Vector2D& position, const Vector2D& size, const Color& Col, float Outline, const Color& OutlineCol)
{
	ProtectedPtr<Shape> shape = CreateShape(name);
	shape->MakeRectangle(position, position + size, Col, Outline, OutlineCol);
	return shape;
}
//------------------------------------------------------------------------------
ProtectedPtr<Shape> Group::CreateShapeCircle(const std::string &name, const Vector2D& Center, float Radius, const Color& Col, float Outline, const Color& OutlineCol)
{
	ProtectedPtr<Shape> shape = CreateShape(name);
	shape->MakeCircle(Center, Radius, Col, Outline, OutlineCol);
	return shape;
}
//------------------------------------------------------------------------------
Object *Group::FindByNameUncast(const std::string &name, const size_t &id)
{
    size_t current = 0;

    for(DrawZPosMap::iterator it = m_drawTbl.begin(); it != m_drawTbl.end(); it++ )
    {
        for(DrawLowZPosMap::iterator it2 = it->second.begin(); it2 != it->second.end(); it2++ )
        {
            if(it2->second->GetName() == name)
            {
                if(id == current)
                {
                    return it2->second;
                }

                current++;
            }
        }
    }

    for(std::vector<PhysicObject*>::iterator it = m_phTbl.begin(); it != m_phTbl.end(); it++)
    {
        if((*it)->GetName() == name)
        {
            if(id == current)
            {
                return *it;
            }

            current++;
        }
    }

    MY_THROW("Group::FindByName() : Object '" << name << "' not found in group '" << GetName() << "'!");
}
//------------------------------------------------------------------------------
/** Get a lua table containing all its childs.\\n
  * Warning: no particular order are warranted.
  */
luabind::object Group::GetChildTable()
{
    luabind::object table = luabind::newtable(Global::GetLuaState());

    size_t current = 1; // Index start at 1 in lua !

    for(DrawZPosMap::iterator it = m_drawTbl.begin(); it != m_drawTbl.end(); it++ )
    {
        for(DrawLowZPosMap::iterator it2 = it->second.begin(); it2 != it->second.end(); it2++ )
        {
            table[current] = CastObject(it2->second);
            current++;
        }
    }

    for(std::vector<PhysicObject*>::iterator it = m_phTbl.begin(); it != m_phTbl.end(); it++)
    {
        table[current] = CastObject(*it);
        current++;
    }

    return table;
}
//------------------------------------------------------------------------------

/** Get a child from its index in the group (a number for 1 to GetChildCount().\\n
  * Warning: no particular order are warranted.
  */
luabind::object Group::FindByIndex(int id)
{
    Object *obj = NULL;
    int current = 1; // Index start at 1 in lua !

    for(DrawZPosMap::iterator it = m_drawTbl.begin(); it != m_drawTbl.end() && obj == NULL; it++ )
    {
        for(DrawLowZPosMap::iterator it2 = it->second.begin(); it2 != it->second.end(); it2++ )
        {
            if(id == current)
            {
                obj = it2->second;
                break;
            }

            current++;
        }
    }

    for(std::vector<PhysicObject*>::iterator it = m_phTbl.begin(); it != m_phTbl.end() && obj == NULL; it++)
    {
        if(id == current)
        {
            obj = *it;
            break;
        }

        current++;
    }

    if(obj == NULL)
    {
        MY_THROW("Group::FindByIndex() : Object not found in group '" << GetName() << "' (id is too high) !");
    }

    // Cast
    return CastObject(obj);
}

//------------------------------------------------------------------------------
luabind::object Group::FindByName(const std::string &name)
{
    Object *obj = NULL;

    try
    {
        obj = FindByNameUncast(name);
    }
    catch(const std::exception &e)
    {
        return luabind::object(); // nil
    }

    return CastObject(obj);
}
//------------------------------------------------------------------------------
void Group::PropagateGlobalRotatedPos(const Vector2D &parentPos, float parentAngle)
{
    Drawable::PropagateGlobalRotatedPos(parentPos, parentAngle);

    for(DrawZPosMap::iterator it = m_drawTbl.begin(); it != m_drawTbl.end(); it++ )
    {
        for(DrawLowZPosMap::iterator it2 = it->second.begin(); it2 != it->second.end(); it2++ )
        {
            it2->second->PropagateGlobalRotatedPos(GetGlobalPosition(), GetGlobalRotation());
        }
    }

    for(std::vector<PhysicObject*>::iterator it = m_phTbl.begin(); it != m_phTbl.end(); it++ )
    {
        (*it)->PropagateGlobalRotatedPos(GetGlobalPosition(), GetGlobalRotation());
    }
}
//------------------------------------------------------------------------------
/*void Group::PropagateRotation(float deltaRotation)
{*/
    /*for(DrawZPosMap::iterator it = m_drawTbl.begin(); it != m_drawTbl.end(); it++ )
    {
        for(DrawLowZPosMap::iterator it2 = it->second.begin(); it2 != it->second.end(); it2++ )
        {
            if(it2->second->GetType() == OBJ_TYPE_DRAWABLE_GROUP)
            {
                (static_cast<Group*>(it2->second))->PropageRotation(deltaRotation);
            }

            it2->second->SetPosition(PointRotate(Vector2D(0, 0), it2->second->GetPosition(), deltaRotation));
        }
    }

    for(std::vector<PhysicObject*>::iterator it = m_phTbl.begin(); it != m_phTbl.end(); it++ )
    {
        (*it)->SetPosition(PointRotate(Vector2D(0, 0), (*it)->GetPosition(), deltaRotation));
    }*/

    /*for(DrawZPosMap::iterator it = m_drawTbl.begin(); it != m_drawTbl.end(); it++ )
    {
        for(DrawLowZPosMap::iterator it2 = it->second.begin(); it2 != it->second.end(); it2++ )
        {
            float dist, angle;
            it2->second->GetPolarPosition(dist, angle);
            it2->second->SetPolarPosition(dist, angle + deltaRotation);

            if(it2->second->GetType() == OBJ_TYPE_DRAWABLE_GROUP)
            {
                (static_cast<Group*>(it2->second))->PropagateRotation(deltaRotation);
            }
        }
    }

    for(std::vector<PhysicObject*>::iterator it = m_phTbl.begin(); it != m_phTbl.end(); it++ )
    {
        float dist, angle;
        (*it)->GetPolarPosition(dist, angle);
        (*it)->SetPolarPosition(dist, angle + deltaRotation);
    }
}*/

//------------------------------------------------------------------------------
/*void Group::SetRotation(float rotation)
{
    float backRotation = GetRotation();
    Object::SetRotation(rotation);

    PropagateRotation(rotation - backRotation);
}*/
//------------------------------------------------------------------------------
/** Set the cut rectangle. You should enable cutting. */
void Group::SetCutRect(const Vector2D &position, const Vector2D &size)
{
    m_cutRect.Left = position.x;
    m_cutRect.Top = position.y;

    m_cutRect.Right = position.x + max(0.f, size.x);
    m_cutRect.Bottom = position.y + max(0.f, size.y);
}
//------------------------------------------------------------------------------
/** Get the cut rectangle's position. */
Vector2D Group::GetCutRectPosition() const
{
    return Vector2D(m_cutRect.Left, m_cutRect.Top);
}
//------------------------------------------------------------------------------
/** Get the cut rectangle's size. */
Vector2D Group::GetCutRectSize() const
{
    return Vector2D(m_cutRect.Right - m_cutRect.Left, m_cutRect.Bottom - m_cutRect.Top);
}
//------------------------------------------------------------------------------
/*void Group::RegisterEventAutoRemoveOnDestroy(EventHandlerBase *event)
{
    m_eventToBeDestroyedOnDelete.push_back(event);
}*/
//------------------------------------------------------------------------------
void Group::SetPreDestroyFunction(const luabind::object &luaFunction)
{
    if(luaFunction.is_valid() && luabind::type(luaFunction) == LUA_TFUNCTION)
    {
        m_luaPreDestroyFunction = luaFunction;
    }
    else
    {
        m_luaPreDestroyFunction = luabind::object();
    }
}
//------------------------------------------------------------------------------
luabind::object Group::GetPreDestroyFunction() const
{
    return m_luaPreDestroyFunction;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
